Merge branch 'Options'
[usenet/yapfaq.git] / yapfaq.pl
index 4f42f4f..0a541c8 100644 (file)
--- a/yapfaq.pl
+++ b/yapfaq.pl
@@ -54,21 +54,32 @@ use strict;
 use Net::NNTP;
 use Date::Calc qw(Add_Delta_YM Add_Delta_Days Delta_Days Today);
 use Fcntl ':flock'; # import LOCK_* constants
+use Getopt::Std;
 my ($TDY, $TDM, $TDD) = Today(); #TD: Today's date
 
+my %Options;
+getopts('hvpdt:f:', \%Options);
+if ($Options{'h'}) {
+  print "$0 v $Version\nUsage: $0 [-hvpd] [-t <newsgroups>] [-f <faq>]\n";
+  exit(0);
+};
+my ($Faq) = $Options{'f'} if ($Options{'f'});
+
 my @Config;
-readconfig (\$ConfigFile, \@Config);
+readconfig (\$ConfigFile, \@Config, \$Faq);
 
 foreach (@Config) { 
   my ($LPD,$LPM,$LPY) = (01, 01, 0001);  #LP: Last posting-date
   my ($NPY,$NPM,$NPD);                   #NP: Next posting-date
   my $SupersedeMID;
   
-  my ($ActName,$File,$PFreq) =($$_{'name'},$$_{'file'},$$_{'posting-frequency'});
+  my ($ActName,$File,$PFreq,$Expire) =($$_{'name'},$$_{'file'},$$_{'posting-frequency'},$$_{'expires'});
   my ($From,$Subject,$NG,$Fup2)=($$_{'from'},$$_{'subject'},$$_{'ngs'},$$_{'fup2'});
   my ($MIDF,$ReplyTo,$ExtHea)=($$_{'mid-format'},$$_{'reply-to'},$$_{'extraheader'});
   my ($Supersede)            =($$_{'supersede'});
-    
+
+  next if (defined($Faq) && $ActName ne $Faq);
+       
   if (open (FH, "<$File.cfg")) {
     while(<FH>){
       if (/##;; Lastpost:\s*(\d{1,2})\.(\d{1,2})\.(\d{2}(\d{2})?)/){
@@ -84,29 +95,36 @@ foreach (@Config) {
 
   $SupersedeMID = "" unless $Supersede;
 
-  if ($PFreq =~ /(\d+)\s*([dw])/) { # Is counted in days or weeks: Use Add_Delta_Days.
-    ($NPY,$NPM,$NPD) = Add_Delta_Days($LPY, $LPM, $LPD, (($2 eq "w")?$1 * 7: $1 * 1));
-  } elsif ($PFreq =~ /(\d+)\s*([my])/) { #Is counted in months or years: Use Add_Delta_YM
-    ($NPY,$NPM,$NPD) = Add_Delta_YM($LPY, $LPM, $LPD, (($2 eq "m")?(0,$1):($1,0)));
-  }
-    
-  if (Delta_Days($NPY,$NPM,$NPD,$TDY,$TDM,$TDD) >= 0 ) {
-    postfaq(\$ActName,\$File,\$From,\$Subject,\$NG,\$Fup2,\$MIDF,\$ExtHea,\$Sender,\$TDY,\$TDM,\$TDD,\$ReplyTo,\$SupersedeMID);
+  ($NPY,$NPM,$NPD) = calcdelta ($LPY,$LPM,$LPD,$PFreq);
+
+  if (Delta_Days($NPY,$NPM,$NPD,$TDY,$TDM,$TDD) >= 0 or ($Options{'p'})) {
+    if($Options{'d'}) {
+         print "$ActName: Would be posted now (but running in simulation mode [$0 -d]).\n" if $Options{'v'};
+       } else {
+      postfaq(\$ActName,\$File,\$From,\$Subject,\$NG,\$Fup2,\$MIDF,\$ExtHea,\$Sender,\$TDY,\$TDM,\$TDD,\$ReplyTo,\$SupersedeMID,\$Expire);
+       }
+  } elsif($Options{'v'}) {
+    print "$ActName: Nothing to do.\n";
   }
 }
 
 exit;
 
 ################################## readconfig ##################################
# Takes a filename and the reference to an array, which will hold hashes with
# the data from $File.
 
 sub readconfig{
-  my ($File, $Config) = @_;
+  my ($File, $Config, $Faq) = @_;
   my ($LastEntry, $Error, $i) = ('','',0);
 
+  if($Options{'v'}) {
+    print "Reading configuration.\n";
+  }
+
   open FH, "<$$File" or die "$0: E: Can't open $$File: $!";
   while (<FH>) {
+    next if (defined($$Faq) && !/^\s*=====\s*$/ && defined($$Config[$i]{'name'}) && $$Config[$i]{'name'} ne $$Faq );
     if (/^(\s*(\S+)\s*=\s*'?(.*?)'?\s*(#.*$|$)|^(.*?)'?\s*(#.*$|$))/ && not /^\s*$/) {
       $LastEntry = lc($2) if $2;
       $$Config[$i]{$LastEntry} .= $3 if $3;  
@@ -120,6 +138,7 @@ sub readconfig{
 
   #Check saved values:
   for $i (0..$i){
+    next if (defined($$Faq) && defined($$Config[$i]{'name'}) && $$Config[$i]{'name'} ne $$Faq );
     unless($$Config[$i]{'from'} =~ /\S+\@(\S+\.)?\S{2,}\.\S{2,}/) {
       $Error .= "E: The From-header for your project \"$$Config[$i]{'name'}\" seems to be incorrect.\n"
     }
@@ -132,20 +151,44 @@ sub readconfig{
     unless($$Config[$i]{'posting-frequency'} =~ /^\s*\d+\s*[dwmy]\s*$/) {
       $Error .= "E: The Posting-frequency for your project \"$$Config[$i]{'name'}\" is invalid.\n"
     }
+    unless($$Config[$i]{'expires'} =~ /^\s*\d+\s*[dwmy]\s*$/) {
+      $$Config[$i]{'expires'} = '3m'; # set default: 3 month
+         warn "$0: W: The Expires for your project \"$$Config[$i]{'name'}\" is invalid - set to 3 month.\n";
+    }
     $Error .= "-" x 25 . "\n" if $Error;
   }
   die $Error if $Error;
 }
 
+################################# calcdelta #################################
+# Takes a date (year,  month and day) and a time period (1d, 1w, 1m, 1y, ...)
+# and adds the latter to the former
+
+sub calcdelta {
+  my ($Year, $Month, $Day, $Period) = @_;
+  my ($NYear, $NMonth, $NDay);
+
+  if ($Period =~ /(\d+)\s*([dw])/) { # Is counted in days or weeks: Use Add_Delta_Days.
+    ($NYear, $NMonth, $NDay) = Add_Delta_Days($Year, $Month, $Day, (($2 eq "w")?$1 * 7: $1 * 1));
+  } elsif ($Period =~ /(\d+)\s*([my])/) { #Is counted in months or years: Use Add_Delta_YM
+    ($NYear, $NMonth, $NDay) = Add_Delta_YM($Year, $Month, $Day, (($2 eq "m")?(0,$1):($1,0)));
+  }
+  return ($NYear, $NMonth, $NDay);
+}
+  
 ################################## postfaq ##################################
 # Takes a filename and many other vars.
 #
 # It reads the data-file $File and then posts the article.
 
 sub postfaq {
-  my ($ActName,$File,$From,$Subject,$NG,$Fup2,$MIDF,$ExtraHeaders,$Sender,$TDY,$TDM,$TDD,$ReplyTo,$Supersedes) = @_;
+  my ($ActName,$File,$From,$Subject,$NG,$Fup2,$MIDF,$ExtraHeaders,$Sender,$TDY,$TDM,$TDD,$ReplyTo,$Supersedes,$Expire) = @_;
   my (@Header,@Body,$MID,$InRealBody,$LastModified);
 
+  if($Options{'v'}) {
+    print "$$ActName: Preparing to post.\n";
+  }
+  
   #Prepare MID:
   $$TDM = ($$TDM < 10 && $$TDM !~ /^0/) ? "0" . $$TDM : $$TDM;
   $$TDD = ($$TDD < 10 && $$TDD !~ /^0/) ? "0" . $$TDD : $$TDD;
@@ -181,17 +224,22 @@ sub postfaq {
   my $year = (1900 + $time[5]);
   my $tz = $time[8] ? " +0200" : " +0100";
   
-  my ($expY,$expM,$expD) = Add_Delta_YM($year, $month, $day, 0, 3);
+  my ($expY,$expM,$expD) = calcdelta ($year,$month,$day,$$Expire);
   my $expmonthN = ("Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec")[$expM-1];
 
   my $date = "$day $monthN $year " . $hh . ":" . $mm . ":" . $ss . $tz;
   my $expdate = "$expD $expmonthN $expY $hh:$mm:$ss$tz";
-
   #Replace %LM by the content of the news.answer-pseudo-header Last-modified:
   if ($LastModified) {
     $$Subject =~ s/\%LM/$LastModified/;
   }
 
+  # Test mode?
+  if($Options{'t'} and $Options{'t'} !~ /console/i) {
+    $$NG = $Options{'t'};
+  }
+
   #Now create the complete Header:
   push @Header, "From: $$From\n";
   push @Header, "Newsgroups: $$NG\n";
@@ -213,8 +261,15 @@ sub postfaq {
 
   my @Article = ($UsePGP)?@{signpgp(\@Header, \@Body)}:(@Header, "\n", @Body);
   
+  if($Options{'v'}) {
+    print "$$ActName: Posting article ...\n";
+  }
   post(\@Article);
 
+  if($Options{'v'}) {
+    print "$$ActName: Save status information.\n";
+  }
+
   open (FH, ">$$File.cfg") or die "$0: E: Can't open $$File.cfg: $!";
   print FH "##;; Lastpost: $day.$month.$year\n";
   print FH "##;; LastMID: $MID\n";
@@ -229,6 +284,14 @@ sub postfaq {
 sub post {
   my ($ArticleR) = @_;
 
+  # Test mode?
+  if(defined($Options{'t'}) and $Options{'t'} =~ /console/i) {
+    print "\n-----BEGIN--------------------------------------------------\n";
+       print @$ArticleR;
+    print "\n------END---------------------------------------------------\n";
+       return;
+  }
+
   my $NewsConnection = Net::NNTP->new($NNTPServer, Reader => 1)
     or die "$0: E: Can't connect to news server '$NNTPServer'!\n";
 
@@ -84,29 +95,36 @@ foreach (@Config) {
 
 ################################## readconfig ##################################
-# Takes a filename and the reference to an array, which will hold hashes with
-# the data from $File.
+# Takes a filename, a reference to an array, which will hold hashes with
+# the data from $File, and - optionally - the name of the (single) FAQ to post
 
 sub readconfig{
This page took 0.014026 seconds and 4 git commands to generate.